#include "font.h"
#include "systick.h"
#include "tftlcd.h"

uint16_t PEN_COLOR = BLACK;   // 画笔颜色
uint16_t BACK_COLOR = WHITE;  // 背景颜色
uint16_t DeviceCode;          // 设备模式

void LCD_RESET() {
    LCD_RST_CLR;
    delay_ms(100);
    LCD_RST_SET;
}

// 写寄存器 -> RS = 0
// what is the difference between set LCD_WR_CLR before and after output
static void LCD_WR_REG(uint8_t reg) {
    LCD_RS_CLR;
    LCD_CS_CLR;
    DATAOUT(reg);
    LCD_WR_CLR;
    LCD_WR_SET;
    LCD_CS_SET;
}

// 写数据 -> RS = 1 - 8bit
static void LCD_WR_DATA(uchar data) {
    LCD_RS_SET;
    LCD_CS_CLR;
    DATAOUT(data);
    LCD_WR_CLR;
    LCD_WR_SET;
    LCD_CS_SET;
}

// 写数据 -> RS = 1 - 16bit
static void LCD_WR_DATA_16BIT(uint16_t data) {
    LCD_WR_DATA(data >> 8);
    LCD_WR_DATA(data & 0xff);
}

// 写寄存器和数据 - 8bit
static void LCD_WR_REG_DATA(uint8_t reg, uchar data) {
    LCD_WR_REG(reg);
    LCD_WR_DATA(data);
}

// 写寄存器和数据 - 16bit
static void LCD_WR_REG_DATA_16BIT(uint8_t reg, uint16_t data) {
    LCD_WR_REG(reg);
    LCD_WR_DATA_16BIT(data);
}

// // 写GRAM prepare
// static void LCD_WR_RAM_Prepare() { LCD_WR_REG(LCD_RAM_WR); }

// // 写GRAM 16-bit
// static void LCD_WR_RAM(uint16_t rgb) {
//     LCD_WR_DATA(rgb >> 8);
//     LCD_WR_DATA(rgb);
// }

// 读寄存器
static uchar LCD_RD_REG(uint8_t reg) {
    uchar tmp;
    LCD_WR_REG(reg);

    for (uint8_t i = 0; i < 8; i++) GPIO_IN(D, i, 1);

    LCD_RS_SET;
    LCD_CS_CLR;
    LCD_RD_CLR;
    tmp = DATAIN;
    LCD_RD_SET;
    LCD_CS_SET;

    for (uint8_t i = 0; i < 8; i++) GPIO_OUT(D, i, 1);

    return tmp;
}

void LCD_Init() {
    // Enable Clock
    SIM->SCGC5 |= SIM_SCGC5_PORTA_MASK     // PTA19 : Back Light (BL)
                  | SIM_SCGC5_PORTB_MASK   // PTB0 - PTB3 : coordinates, PTB8 - PTB11 : RS, CS, RD, WR
                  | SIM_SCGC5_PORTC_MASK   // PTC16 - PTC17 : X-Y In
                  | SIM_SCGC5_PORTD_MASK   // PTD0 - PTD7: data
                  | SIM_SCGC5_PORTE_MASK;  // PTE31 : ReSeT (RST)

    // Set GPIO
    GPIO_OUT(A, 19, 1);  // Back Light
    GPIO_OUT(E, 31, 1);  // ReSeT (RST)
    GPIO_OUT(B, 16, 1);  // Y-IN
    GPIO_OUT(B, 17, 1);  // X-IN
    for (uint8_t i = 0; i < 4; i++) GPIO_OUT(B, i, 1);
    for (uint8_t i = 8; i < 12; i++) GPIO_OUT(B, i, 1);
    for (uint8_t i = 0; i < 8; i++) GPIO_OUT(D, i, 1);

    LCD_RESET();
    delay_ms(50);
    LCD_WR_REG_DATA(0x00, 0x0001);  // Software Reset
    delay_ms(50);
    DeviceCode = LCD_RD_REG(0x04);  // Read display identification information 0x04

    // Driver
    LCD_WR_REG(0xCF);
    LCD_WR_DATA(0x00);
    LCD_WR_DATA(0xD9);
    LCD_WR_DATA(0X30);

    LCD_WR_REG(0xED);
    LCD_WR_DATA(0x64);
    LCD_WR_DATA(0x03);
    LCD_WR_DATA(0X12);
    LCD_WR_DATA(0X81);

    LCD_WR_REG(0xE8);
    LCD_WR_DATA(0x85);
    LCD_WR_DATA(0x10);
    LCD_WR_DATA(0x78);

    LCD_WR_REG(0xCB);
    LCD_WR_DATA(0x39);
    LCD_WR_DATA(0x2C);
    LCD_WR_DATA(0x00);
    LCD_WR_DATA(0x34);
    LCD_WR_DATA(0x02);

    LCD_WR_REG(0xF7);
    LCD_WR_DATA(0x20);

    LCD_WR_REG(0xEA);
    LCD_WR_DATA(0x00);
    LCD_WR_DATA(0x00);

    LCD_WR_REG(0xC0);   // Power control
    LCD_WR_DATA(0x21);  // VRH[5:0]

    LCD_WR_REG(0xC1);   // Power control
    LCD_WR_DATA(0x12);  // SAP[2:0];BT[3:0]

    LCD_WR_REG(0xC5);  // VCM control
    LCD_WR_DATA(0x32);
    LCD_WR_DATA(0x3C);

    LCD_WR_REG(0xC7);  // VCM control2
    LCD_WR_DATA(0XC1);

    LCD_WR_REG(LCD_RAM_AC);
#if USE_HORIZONTAL == 1
    LCD_WR_DATA(0x88);  // 0x08: MY = 1, MX = MV = 0 从左到右 从下到下, BGR = 1s
#else
    LCD_WR_DATA(0x08);  // 0x08: MY = MX = MV = 0 从左到右 从上到下, BGR = 1s
#endif

    LCD_WR_REG(0x3A);
    LCD_WR_DATA(0x55);

    LCD_WR_REG(0xB1);
    LCD_WR_DATA(0x00);
    LCD_WR_DATA(0x18);

    LCD_WR_REG(0xB6);  // Display Function Control
    LCD_WR_DATA(0x0A);
    LCD_WR_DATA(0xA2);

    LCD_WR_REG(0xF2);  // 3Gamma Function Disable
    LCD_WR_DATA(0x00);

    LCD_WR_REG(0x26);  // Gamma curve selected
    LCD_WR_DATA(0x01);

    LCD_WR_REG(0xE0);  // Set Gamma
    LCD_WR_DATA(0x0F);
    LCD_WR_DATA(0x20);
    LCD_WR_DATA(0x1E);
    LCD_WR_DATA(0x09);
    LCD_WR_DATA(0x12);
    LCD_WR_DATA(0x0B);
    LCD_WR_DATA(0x50);
    LCD_WR_DATA(0XBA);
    LCD_WR_DATA(0x44);
    LCD_WR_DATA(0x09);
    LCD_WR_DATA(0x14);
    LCD_WR_DATA(0x05);
    LCD_WR_DATA(0x23);
    LCD_WR_DATA(0x21);
    LCD_WR_DATA(0x00);

    LCD_WR_REG(0XE1);  // Set Gamma
    LCD_WR_DATA(0x00);
    LCD_WR_DATA(0x19);
    LCD_WR_DATA(0x19);
    LCD_WR_DATA(0x00);
    LCD_WR_DATA(0x12);
    LCD_WR_DATA(0x07);
    LCD_WR_DATA(0x2D);
    LCD_WR_DATA(0x28);
    LCD_WR_DATA(0x3F);
    LCD_WR_DATA(0x02);
    LCD_WR_DATA(0x0A);
    LCD_WR_DATA(0x08);
    LCD_WR_DATA(0x25);
    LCD_WR_DATA(0x2D);
    LCD_WR_DATA(0x0F);

    LCD_WR_REG(0x11);  // Exit Sleep
    delay_ms(120);
    LCD_WR_REG(0x29);  // Display on

    LCD_BL_CLR;  // Back Light On
    LCD_Clear();
}

void LCD_SetColor(uint16_t color) { PEN_COLOR = color; }

void LCD_SetBackColor(uint16_t color) { BACK_COLOR = color; }

void LCD_SetXY(uint16_t x, uint16_t y) {
#if USE_HORIZONTAL == 1
    LCD_WR_REG_DATA_16BIT(LCD_PAGE_WR, x);
    LCD_WR_REG_DATA_16BIT(LCD_COL_WR, y);
#else
    LCD_WR_REG_DATA_16BIT(LCD_COL_WR, x);
    LCD_WR_REG_DATA_16BIT(LCD_PAGE_WR, y);
#endif
}

// Set a rectangle area
void LCD_SetRec(uint16_t xs, uint16_t xe, uint16_t ys, uint16_t ye) {
#if USE_HORIZONTAL == 1
    LCD_WR_REG(LCD_PAGE_WR);
    LCD_WR_DATA_16BIT(xs);
    LCD_WR_DATA_16BIT(xe);

    LCD_WR_REG(LCD_COL_WR);
    LCD_WR_DATA_16BIT(ys);
    LCD_WR_DATA_16BIT(ye);
#else
    LCD_WR_REG(LCD_COL_WR);
    LCD_WR_DATA_16BIT(xs);
    LCD_WR_DATA_16BIT(xe);

    LCD_WR_REG(LCD_PAGE_WR);
    LCD_WR_DATA_16BIT(ys);
    LCD_WR_DATA_16BIT(ye);
#endif
}

void LCD_DrawPoint(uint16_t x, uint16_t y, uint16_t color) {
    LCD_SetXY(x, y);
    LCD_WR_REG_DATA_16BIT(LCD_RAM_WR, color);
}

void LCD_DrawLine(uint16_t x1, uint16_t y1, uint16_t x2, uint16_t y2) {
    int16_t Dx, Dy;
    uint16_t x, y, d, dx, dy, xe, ye;

    xe = ye = 0;
    x = x1, y = y1;
    Dx = x2 - x1, Dy = y2 - y1;

    if (Dx > 0) dx = 1;
    else if (Dx == 0) dx = 0;
    else {
        dx = -1;
        Dx = -Dx;
    }
    if (Dy > 0) dy = 1;
    else if (Dy == 0) dy = 0;
    else {
        dy = -1;
        Dy = -Dy;
    }
    if (Dx > Dy) d = Dx;
    else d = Dy;

    for (uint16_t i = 0; i <= d + 1; i++)  // 画线输出
    {
        LCD_DrawPoint(x, y, PEN_COLOR);  // 画点
        xe += Dx, ye += Dy;
        if (xe > d) xe -= d, x += dx;
        if (ye > d) ye -= d, y += dy;
    }
}

void LCD_Clear() {
    LCD_SetRec(0, LCD_W - 1, 0, LCD_H - 1);
    LCD_WR_REG(LCD_RAM_WR);
    for (uint32_t i = 0; i < LCD_W * LCD_H; i++) LCD_WR_DATA_16BIT(BACK_COLOR);
    // for (uint16_t i = 0; i < LCD_W; i++)
    //     for (uint16_t j = 0; j < LCD_H; j++) {
    //         LCD_SetXY(i, j);
    //         LCD_WR_REG_DATA_16BIT(LCD_RAM_WR, BACK_COLOR);
    //     }
}

/*
    @brief Display a charactor on the position of (x,y)
    @param x x of the lower left of the character
    @param y y of the lower left of the character
    @param sign the charactor
    @param size size of the charactor, 12x06 or 16x08
    @param mode 0: non-overlap, 1: overlap
*/
void LCD_ShowChar(uint16_t x, uint16_t y, uint8_t sign, uint8_t size, uint8_t mode) {
    if (x > LCD_W || y > LCD_H) return;

    uchar tmp;
    const uchar *font;
    if (size == 12) font = &asc2_1206[0][0];
    else font = &asc2_1608[0][0];

    sign = sign - ' ';
    if (!mode) {  // non-overlap
        for (uint16_t py = 0; py < size; py++) {
            tmp = font[sign * size + py];
            for (uint16_t px = 0; px < size / 2; px++) {
                if (tmp & 0x01) LCD_DrawPoint(x + px, y + py, PEN_COLOR);
                else LCD_DrawPoint(x + px, y + py, BACK_COLOR);
                tmp >>= 1;
            }
        }
    } else  // overlap
    {
        for (uint16_t py = 0; py < size; py++) {
            tmp = font[sign * size + py];
            for (uint16_t px = 0; px < size / 2; px++) {
                if (tmp & 0x01) LCD_DrawPoint(x + px, y + py, PEN_COLOR);
                tmp >>= 1;
            }
        }
    }
}

/*
    @brief Display a number on the position of (x,y)
    @param x x of the first charactor
    @param y y of the first charactor
    @param num the number
    @param len length of the number
    @param size size of one charactor, 12x06 or 16x08
    @param dir the direction to display the number, 0: keep x the same, 1: keep y the same
    @param mode 0: non-overlap, 1: overlap
*/
void LCD_ShowNum(uint16_t x, uint16_t y, uint32_t num, uint8_t len, uint8_t size, uint8_t dir, uint8_t mode) {
    uint8_t tmp;
    if (!dir)
        for (uint8_t i = 0; i < len; i++) {
            tmp = num % 10;
            LCD_ShowChar(x + size / 2 * (len - i - 1), y, tmp + '0', size, mode);
            num /= 10;
        }
    else
        for (uint8_t i = 0; i < len; i++) {
            tmp = num % 10;
            LCD_ShowChar(x, y + size * (len - i - 1), tmp + '0', size, mode);
            num /= 10;
        }
}

/*
    @brief Display a string on the position of (x,y)
    @param x x of the first charactor
    @param y y of the first charactor
    @param str the string
    @param size size of one charactor, 12x06 or 16x08
    @param dir the direction to display the string, 0: keep x the same, 1: keep y the same
    @param mode 0: non-overlap, 1: overlap
*/
void LCD_ShowString(uint16_t x, uint16_t y, char *str, uint8_t size, uint8_t dir, uint8_t mode) {
    if (!dir)
        while (*str) {
            LCD_ShowChar(x, y, *str++, size, mode);
            x += size / 2;
        }
    else
        while (*str) {
            LCD_ShowChar(x, y, *str++, size, mode);
            y += size;
        }
}
